package jdk.internal.org.objectweb.asm.util;

import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Attribute;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public final class TraceMethodVisitor extends MethodVisitor
{
    public final Printer p;
    
    public TraceMethodVisitor(Printer printer) {
        this(null, printer);
    }
    
    public TraceMethodVisitor(MethodVisitor methodVisitor, Printer p2) {
        super(327680, methodVisitor);
        this.p = p2;
    }
    
    @Override
    public void visitParameter(String name, int access) {
        this.p.visitParameter(name, access);
        super.visitParameter(name, access);
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitAnnotation(descriptor, visible), this.p.visitMethodAnnotation(descriptor, visible));
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int typeRef, TypePath typePath, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitTypeAnnotation(typeRef, typePath, descriptor, visible), this.p.visitMethodTypeAnnotation(typeRef, typePath, descriptor, visible));
    }
    
    @Override
    public void visitAttribute(Attribute attribute) {
        this.p.visitMethodAttribute(attribute);
        super.visitAttribute(attribute);
    }
    
    @Override
    public AnnotationVisitor visitAnnotationDefault() {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitAnnotationDefault(), this.p.visitAnnotationDefault());
    }
    
    @Override
    public AnnotationVisitor visitParameterAnnotation(int parameter, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitParameterAnnotation(parameter, descriptor, visible), this.p.visitParameterAnnotation(parameter, descriptor, visible));
    }
    
    @Override
    public void visitCode() {
        this.p.visitCode();
        super.visitCode();
    }
    
    @Override
    public void visitFrame(int type, int numLocal, Object[] local, int numStack, Object[] stack) {
        this.p.visitFrame(type, numLocal, local, numStack, stack);
        super.visitFrame(type, numLocal, local, numStack, stack);
    }
    
    @Override
    public void visitInsn(int opcode) {
        this.p.visitInsn(opcode);
        super.visitInsn(opcode);
    }
    
    @Override
    public void visitIntInsn(int opcode, int operand) {
        this.p.visitIntInsn(opcode, operand);
        super.visitIntInsn(opcode, operand);
    }
    
    @Override
    public void visitVarInsn(int opcode, int varIndex) {
        this.p.visitVarInsn(opcode, varIndex);
        super.visitVarInsn(opcode, varIndex);
    }
    
    @Override
    public void visitTypeInsn(int opcode, String type) {
        this.p.visitTypeInsn(opcode, type);
        super.visitTypeInsn(opcode, type);
    }
    
    @Override
    public void visitFieldInsn(int opcode, String owner, String name, String descriptor) {
        this.p.visitFieldInsn(opcode, owner, name, descriptor);
        super.visitFieldInsn(opcode, owner, name, descriptor);
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3) {
        if (this.api >= 327680) {
            super.visitMethodInsn(n, s, s2, s3);
            return;
        }
        this.p.visitMethodInsn(n, s, s2, s3);
        if (this.mv != null) {
            this.mv.visitMethodInsn(n, s, s2, s3);
        }
        return;
    }
    
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        if (this.api < 327680) {
            super.visitMethodInsn(n, s, s2, s3, b);
            return;
        }
        this.p.visitMethodInsn(n, s, s2, s3, b);
        if (this.mv != null) {
            this.mv.visitMethodInsn(n, s, s2, s3, b);
        }
        return;
    }
    
    @Override
    public void visitInvokeDynamicInsn(String name, String descriptor, Handle bootstrapMethodHandle, Object... bootstrapMethodArguments) {
        this.p.visitInvokeDynamicInsn(name, descriptor, bootstrapMethodHandle, bootstrapMethodArguments);
        super.visitInvokeDynamicInsn(name, descriptor, bootstrapMethodHandle, bootstrapMethodArguments);
    }
    
    @Override
    public void visitJumpInsn(int opcode, Label label) {
        this.p.visitJumpInsn(opcode, label);
        super.visitJumpInsn(opcode, label);
    }
    
    @Override
    public void visitLabel(Label label) {
        this.p.visitLabel(label);
        super.visitLabel(label);
    }
    
    @Override
    public void visitLdcInsn(Object value) {
        this.p.visitLdcInsn(value);
        super.visitLdcInsn(value);
    }
    
    @Override
    public void visitIincInsn(int varIndex, int increment) {
        this.p.visitIincInsn(varIndex, increment);
        super.visitIincInsn(varIndex, increment);
    }
    
    @Override
    public void visitTableSwitchInsn(int min, int max, Label dflt, Label... labels) {
        this.p.visitTableSwitchInsn(min, max, dflt, labels);
        super.visitTableSwitchInsn(min, max, dflt, labels);
    }
    
    @Override
    public void visitLookupSwitchInsn(Label dflt, int[] keys, Label[] labels) {
        this.p.visitLookupSwitchInsn(dflt, keys, labels);
        super.visitLookupSwitchInsn(dflt, keys, labels);
    }
    
    @Override
    public void visitMultiANewArrayInsn(String descriptor, int numDimensions) {
        this.p.visitMultiANewArrayInsn(descriptor, numDimensions);
        super.visitMultiANewArrayInsn(descriptor, numDimensions);
    }
    
    @Override
    public AnnotationVisitor visitInsnAnnotation(int typeRef, TypePath typePath, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitInsnAnnotation(typeRef, typePath, descriptor, visible), this.p.visitInsnAnnotation(typeRef, typePath, descriptor, visible));
    }
    
    @Override
    public void visitTryCatchBlock(Label start, Label end, Label handler, String type) {
        this.p.visitTryCatchBlock(start, end, handler, type);
        super.visitTryCatchBlock(start, end, handler, type);
    }
    
    @Override
    public AnnotationVisitor visitTryCatchAnnotation(int typeRef, TypePath typePath, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitTryCatchAnnotation(typeRef, typePath, descriptor, visible), this.p.visitTryCatchAnnotation(typeRef, typePath, descriptor, visible));
    }
    
    @Override
    public void visitLocalVariable(String name, String descriptor, String signature, Label start, Label end, int index) {
        this.p.visitLocalVariable(name, descriptor, signature, start, end, index);
        super.visitLocalVariable(name, descriptor, signature, start, end, index);
    }
    
    @Override
    public AnnotationVisitor visitLocalVariableAnnotation(int typeRef, TypePath typePath, Label[] start, Label[] end, int[] index, String descriptor, boolean visible) {
        return new TraceAnnotationVisitor((this.mv == null) ? null : this.mv.visitLocalVariableAnnotation(typeRef, typePath, start, end, index, descriptor, visible), this.p.visitLocalVariableAnnotation(typeRef, typePath, start, end, index, descriptor, visible));
    }
    
    @Override
    public void visitLineNumber(int line, Label start) {
        this.p.visitLineNumber(line, start);
        super.visitLineNumber(line, start);
    }
    
    @Override
    public void visitMaxs(int maxStack, int maxLocals) {
        this.p.visitMaxs(maxStack, maxLocals);
        super.visitMaxs(maxStack, maxLocals);
    }
    
    @Override
    public void visitEnd() {
        this.p.visitMethodEnd();
        super.visitEnd();
    }
}
